# Generated by Django 3.2.2 on 2022-03-14 09:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('xperms', '0004_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='description',
            field=models.CharField(blank=True, default='', max_length=180),
        ),
    ]
