# Generated by Django 2.2.15 on 2020-09-16 18:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('xperms', '0003_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='perm',
            name='groups',
        ),
    ]
