# Generated by Django 4.1 on 2022-08-11 13:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('xperms', '0005_group_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='perm',
            name='type',
            field=models.CharField(choices=[('generic', 'generic'), ('model', 'model'), ('object', 'object'), ('field', 'field')], default='generic', max_length=10),
        ),
    ]
