# Generated by Django 2.0 on 2018-07-13 14:50

from django.db import migrations, models
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('xperms', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='perm',
            options={'base_manager_name': 'related_manager'},
        ),
        migrations.AlterModelManagers(
            name='perm',
            managers=[
                ('related_manager', django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterField(
            model_name='perm',
            name='object_id',
            field=models.TextField(blank=True, null=True, verbose_name='object pk'),
        ),
        migrations.AlterUniqueTogether(
            name='perm',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='perm',
            name='field_name',
        ),
    ]
